// Package csv provides stream abstractions that allow the fields of a CSV record to be addressed by the values of the fields of the stream's header record.
//
// It provides an alternative to the stream abstractions in encoding/csv which use unnamed string slices as the stream record type.
package csv
